"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var user_dao_1 = require("../dao/user-dao");
var group_dao_1 = require("../dao/group-dao");
var directory_services_dao_1 = require("../dao/directory-services-dao");
var account_systems_dao_1 = require("../dao/account-systems-dao");
var directoryservice_config_dao_1 = require("../dao/directoryservice-config-dao");
var abstract_repository_1 = require("./abstract-repository");
var directory_dao_1 = require("../dao/directory-dao");
var model_event_name_1 = require("../model-event-name");
var model_1 = require("../model");
var datastore_service_1 = require("../service/datastore-service");
var _ = require("lodash");
var AccountRepository = (function (_super) {
    __extends(AccountRepository, _super);
    function AccountRepository(userDao, groupDao, datastoreService, directoryServiceDao, directoryserviceConfigDao, directoryDao, accountSystemDao) {
        var _this = _super.call(this, [
            model_1.Model.User,
            model_1.Model.Group,
            model_1.Model.Directory
        ]) || this;
        _this.userDao = userDao;
        _this.groupDao = groupDao;
        _this.datastoreService = datastoreService;
        _this.directoryServiceDao = directoryServiceDao;
        _this.directoryserviceConfigDao = directoryserviceConfigDao;
        _this.directoryDao = directoryDao;
        _this.accountSystemDao = accountSystemDao;
        return _this;
    }
    AccountRepository.getInstance = function () {
        if (!AccountRepository.instance) {
            AccountRepository.instance = new AccountRepository(new user_dao_1.UserDao(), new group_dao_1.GroupDao(), datastore_service_1.DatastoreService.getInstance(), new directory_services_dao_1.DirectoryServicesDao(), new directoryservice_config_dao_1.DirectoryserviceConfigDao(), new directory_dao_1.DirectoryDao(), new account_systems_dao_1.AccountSystemDao());
        }
        return AccountRepository.instance;
    };
    AccountRepository.prototype.loadUsers = function () {
        return this.userDao.list();
    };
    AccountRepository.prototype.loadGroups = function () {
        return this.groupDao.list();
    };
    AccountRepository.prototype.listSystemUsersAndGroups = function () {
        return Promise.all([
            this.listSystemUsers(),
            this.listSystemGroups()
        ]).spread(function (users, groups) { return _.concat(users, groups); });
    };
    //@deprecated
    AccountRepository.prototype.listUsers = function () {
        return this.users ? Promise.resolve(this.users.toSet().toJS()) : this.userDao.list();
    };
    AccountRepository.prototype.listLocalUsers = function () {
        return this.userDao.stream(false, { origin: { directory: 'local' } }).then(function (stream) {
            return stream.get('data').toJS();
            ;
        });
    };
    AccountRepository.prototype.listSystemUsers = function () {
        if (this.systemUsersPromise) {
            return this.systemUsersPromise;
        }
        return this.systemUsersPromise = this.userDao.list(false, { builtin: true });
    };
    AccountRepository.prototype.streamUsers = function () {
        var _this = this;
        var promise;
        if (this.usersStreamId) {
            promise = Promise.resolve(this.datastoreService.getState().get('streams').get(this.usersStreamId));
        }
        else {
            promise = this.userDao.stream(true, { builtin: false });
        }
        return promise.then(function (stream) {
            var dataArray = stream.get('data').toJS();
            _this.userDao.register();
            _this.usersStreamId = stream.get('streamId');
            dataArray._objectType = _this.userDao.objectType;
            // FIXME!!
            // DTM montage
            dataArray._stream = stream;
            return dataArray;
        });
    };
    AccountRepository.prototype.getUserEmptyList = function () {
        return this.userDao.getEmptyList();
    };
    AccountRepository.prototype.getGroupEmptyList = function () {
        return this.groupDao.getEmptyList();
    };
    AccountRepository.prototype.getDirectoryServicesEmptyList = function () {
        return this.directoryServiceDao.getEmptyList();
    };
    AccountRepository.prototype.getAccountSystemEmptyList = function () {
        return this.accountSystemDao.getEmptyList();
    };
    // need discussion
    AccountRepository.prototype.getNextSequenceForStream = function (streamId) {
        return this.groupDao.getNextSequenceForStream(streamId);
    };
    AccountRepository.prototype.findUserWithName = function (name) {
        return this.userDao.findSingleEntry({ username: name });
    };
    AccountRepository.prototype.saveUser = function (user) {
        return this.userDao.save(user);
    };
    // @deprecated
    AccountRepository.prototype.listGroups = function () {
        return this.groups ? Promise.resolve(this.groups.toSet().toJS()) : this.groupDao.list();
    };
    AccountRepository.prototype.listLocalGroups = function () {
        return this.groupDao.stream(false, { origin: { directory: 'local' } }).then(function (stream) {
            return stream.get('data').toJS();
            ;
        });
    };
    AccountRepository.prototype.listSystemGroups = function () {
        if (this.systemGroupsPromise) {
            return this.systemGroupsPromise;
        }
        return this.systemGroupsPromise = this.groupDao.list(false, { builtin: true });
    };
    //TODO: ask only ids? (improvements)
    AccountRepository.prototype.streamGroups = function () {
        var _this = this;
        var promise;
        if (this.groupsStreamId) {
            promise = Promise.resolve(this.datastoreService.getState().get("streams").get(this.groupsStreamId));
        }
        else {
            promise = this.groupDao.stream(true, { builtin: false });
        }
        return promise.then(function (stream) {
            var dataArray = stream.get('data').toJS();
            _this.groupDao.register();
            _this.groupsStreamId = stream.get('streamId');
            dataArray._objectType = _this.groupDao.objectType;
            // FIXME!!
            // DTM montage
            dataArray._stream = stream;
            return dataArray;
        });
    };
    AccountRepository.prototype.getNextUid = function () {
        return this.userDao.getNextUid();
    };
    AccountRepository.prototype.getNewUser = function () {
        return this.userDao.getNewInstance();
    };
    AccountRepository.prototype.getNewGroup = function () {
        return this.groupDao.getNewInstance();
    };
    AccountRepository.prototype.getNewDirectoryServices = function () {
        return this.directoryServiceDao.getNewInstance();
    };
    AccountRepository.prototype.getDirectoryServiceConfig = function () {
        return this.directoryserviceConfigDao.get();
    };
    AccountRepository.prototype.listDirectories = function () {
        var promise = this.directories ? Promise.resolve(this.directories.valueSeq().toJS()) : this.directoryDao.list();
        return promise.then(function (directories) {
            _.forEach(directories, function (directory) {
                directory.label = AccountRepository.DIRECTORY_TYPES_LABELS[directory.type];
            });
            return directories;
        });
    };
    AccountRepository.prototype.getNewDirectoryForType = function (type) {
        return this.directoryDao.getNewInstance().then(function (directory) {
            directory.type = type;
            directory._tmpId = type;
            directory.parameters = { '%type': type + '-directory-params' };
            directory.label = AccountRepository.DIRECTORY_TYPES_LABELS[type];
            return directory;
        });
    };
    AccountRepository.prototype.searchUser = function (value) {
        return this.searchUserWithCriteria({ username: [['match', value + "*"]] });
    };
    AccountRepository.prototype.searchGroup = function (value) {
        return this.searchGroupWithCriteria({ name: [['match', value + "*"]] });
    };
    AccountRepository.prototype.searchGroupWithCriteria = function (criteria) {
        return this.groupDao.stream(false, criteria).then(function (results) {
            return results.get('data').toJS();
        });
    };
    AccountRepository.prototype.searchUserWithCriteria = function (criteria) {
        return this.userDao.stream(false, criteria).then(function (results) {
            return results.get('data').toJS();
        });
    };
    AccountRepository.prototype.handleStateChange = function (name, state) {
        switch (name) {
            case model_1.Model.User:
                this.users = this.dispatchModelEvents(this.users, model_event_name_1.ModelEventName.User, state);
                break;
            case model_1.Model.Group:
                this.groups = this.dispatchModelEvents(this.groups, model_event_name_1.ModelEventName.Group, state);
                break;
            case model_1.Model.Directory:
                this.directories = this.dispatchModelEvents(this.directories, model_event_name_1.ModelEventName.Directory, state);
                break;
            default:
                break;
        }
    };
    AccountRepository.prototype.handleEvent = function (name, data) {
    };
    return AccountRepository;
}(abstract_repository_1.AbstractRepository));
AccountRepository.DIRECTORY_TYPES_LABELS = {
    winbind: 'Active Directory',
    freeipa: 'FreeIPA',
    ldap: 'LDAP',
    nis: 'NIS'
};
exports.AccountRepository = AccountRepository;
