"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var abstract_repository_1 = require("./abstract-repository");
var model_event_name_1 = require("../model-event-name");
var network_interface_dao_1 = require("../dao/network-interface-dao");
var network_config_dao_1 = require("../dao/network-config-dao");
var network_route_dao_1 = require("../dao/network-route-dao");
var network_host_dao_1 = require("../dao/network-host-dao");
var ipmi_dao_1 = require("../dao/ipmi-dao");
var model_1 = require("../model");
var NetworkRepository = (function (_super) {
    __extends(NetworkRepository, _super);
    function NetworkRepository(networkInterfaceDao, networkRouteDao, networkHostDao, networkConfigDao, ipmiDao) {
        var _this = _super.call(this, [
            model_1.Model.NetworkInterface,
            model_1.Model.Ipmi
        ]) || this;
        _this.networkInterfaceDao = networkInterfaceDao;
        _this.networkRouteDao = networkRouteDao;
        _this.networkHostDao = networkHostDao;
        _this.networkConfigDao = networkConfigDao;
        _this.ipmiDao = ipmiDao;
        return _this;
    }
    NetworkRepository.getInstance = function () {
        if (!NetworkRepository.instance) {
            NetworkRepository.instance = new NetworkRepository(new network_interface_dao_1.NetworkInterfaceDao(), new network_route_dao_1.NetworkRouteDao(), new network_host_dao_1.NetworkHostDao(), new network_config_dao_1.NetworkConfigDao(), new ipmi_dao_1.IpmiDao());
        }
        return NetworkRepository.instance;
    };
    NetworkRepository.prototype.listNetworkInterfaces = function () {
        return this.interfaces ? Promise.resolve(this.interfaces.valueSeq().toJS()) : this.networkInterfaceDao.list();
    };
    NetworkRepository.prototype.saveNetworkInterface = function (networkInterface) {
        return this.networkInterfaceDao.save(networkInterface);
    };
    NetworkRepository.prototype.getNewNetworkInterface = function () {
        return this.networkInterfaceDao.getNewInstance();
    };
    NetworkRepository.prototype.listNetworkStaticRoutes = function () {
        return this.networkRouteDao.list();
    };
    NetworkRepository.prototype.getNewNetworkStaticRoute = function () {
        return this.networkRouteDao.getNewInstance();
    };
    NetworkRepository.prototype.saveNetworkStaticRoute = function (route) {
        return this.networkRouteDao.save(route);
    };
    NetworkRepository.prototype.deleteNetworkStaticRoute = function (route) {
        return this.networkRouteDao.delete(route);
    };
    NetworkRepository.prototype.listNetworkHosts = function () {
        return this.networkHostDao.list();
    };
    NetworkRepository.prototype.getNewNetworkHost = function () {
        return this.networkHostDao.getNewInstance();
    };
    NetworkRepository.prototype.saveNetworkHost = function (host) {
        return this.networkHostDao.save(host);
    };
    NetworkRepository.prototype.deleteNetworkHost = function (host) {
        return this.networkHostDao.delete(host);
    };
    NetworkRepository.prototype.listIpmiChannels = function () {
        return this.ipmis ? Promise.resolve(this.ipmis.valueSeq().toJS()) : this.ipmiDao.list();
    };
    NetworkRepository.prototype.getNetworkOverview = function () {
        return Promise.all([
            this.networkInterfaceDao.list(),
            this.networkConfigDao.get()
        ]).spread(function (interfaces, config) { return ({
            interfaces: interfaces,
            config: config
        }); });
    };
    NetworkRepository.prototype.getNetworkSettings = function () {
        return this.networkConfigDao.get().then(function (config) { return ({ config: config }); });
    };
    NetworkRepository.prototype.saveNetworkSettings = function (settings) {
        return this.networkConfigDao.save(settings);
    };
    NetworkRepository.prototype.getMyIps = function () {
        return this.networkConfigDao.getMyIps();
    };
    NetworkRepository.prototype.isIpmiLoaded = function () {
        return this.ipmiDao.isIpmiLoaded();
    };
    NetworkRepository.prototype.isClientInterface = function (networkInterface) {
        var _this = this;
        return this.clientInterfaceId ?
            Promise.resolve(networkInterface.id === this.clientInterfaceId) :
            this.networkConfigDao.getClientInterface().then(function (clientInterface) {
                _this.clientInterfaceId = clientInterface;
                return networkInterface.id === clientInterface;
            });
    };
    NetworkRepository.prototype.handleStateChange = function (name, state) {
        switch (name) {
            case model_1.Model.NetworkInterface:
                this.interfaces = this.dispatchModelEvents(this.interfaces, model_event_name_1.ModelEventName.NetworkInterface, state);
                break;
            case model_1.Model.Ipmi:
                this.ipmis = this.dispatchModelEvents(this.ipmis, model_event_name_1.ModelEventName.Ipmi, state);
                break;
            default:
                break;
        }
    };
    NetworkRepository.prototype.handleEvent = function (name, data) {
    };
    return NetworkRepository;
}(abstract_repository_1.AbstractRepository));
NetworkRepository.INTERFACE_TYPES = {
    VLAN: {
        type: 'vlan',
        label: 'VLAN',
        properties: {
            objectType: 'NetworkInterfaceVlanProperties',
            type: 'network-interface-vlan-properties'
        }
    },
    LAGG: {
        type: 'lagg',
        label: 'LAGG',
        properties: {
            objectType: 'NetworkInterfaceLaggProperties',
            type: 'network-interface-lagg-properties'
        }
    },
    BRIDGE: {
        type: 'bridge',
        label: 'BRIDGE',
        properties: {
            objectType: 'NetworkInterfaceBridgeProperties',
            type: 'network-interface-bridge-properties'
        }
    }
};
exports.NetworkRepository = NetworkRepository;
