"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var share_repository_1 = require("../repository/share-repository");
var volume_repository_1 = require("../repository/volume-repository");
var model_event_name_1 = require("../model-event-name");
var abstract_route_1 = require("./abstract-route");
var model_1 = require("../model");
var _ = require("lodash");
var ShareRoute = (function (_super) {
    __extends(ShareRoute, _super);
    function ShareRoute(shareRepository, volumeRepository) {
        var _this = _super.call(this) || this;
        _this.shareRepository = shareRepository;
        _this.volumeRepository = volumeRepository;
        return _this;
    }
    ShareRoute.getInstance = function () {
        if (!ShareRoute.instance) {
            ShareRoute.instance = new ShareRoute(share_repository_1.ShareRepository.getInstance(), volume_repository_1.VolumeRepository.getInstance());
        }
        return ShareRoute.instance;
    };
    ShareRoute.prototype.list = function (volumeId, stack) {
        var columnIndex = 2, volumePrefix = volumeId + '/', mountPrefix = '/mnt/' + volumePrefix, shareFilter = function (share) {
            return _.startsWith(share.target_path + '/', volumePrefix) ||
                _.startsWith(share.target_path + '/', mountPrefix);
        };
        return this.loadListInColumn(stack, columnIndex, columnIndex - 1, '/share', model_1.Model.Share, this.shareRepository.listShares(), {
            filter: shareFilter,
            sort: 'name'
        });
    };
    ShareRoute.prototype.get = function (volumeId, shareId, stack) {
        var self = this, objectType = model_1.Model.Share;
        return Promise.all([
            this.volumeRepository.listVolumes(),
            this.shareRepository.listShares(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (volumes, shares, uiDescriptor) {
            var columnIndex = 3;
            while (stack.length > columnIndex) {
                var context = stack.pop();
                if (context && context.changeListener) {
                    self.eventDispatcherService.removeEventListener(model_event_name_1.ModelEventName[context.objectType].listChange, context.changeListener);
                }
            }
            var share = _.find(shares, { id: shareId });
            share._volume = _.find(volumes, { id: volumeId });
            stack.push({
                object: share,
                userInterfaceDescriptor: uiDescriptor,
                columnIndex: columnIndex,
                objectType: objectType,
                parentContext: stack[columnIndex - 1],
                path: stack[columnIndex - 1].path + '/_/' + encodeURIComponent(shareId)
            });
            return stack;
        });
    };
    ShareRoute.prototype.selectNewType = function (volumeId, stack) {
        var columnIndex = 3;
        var self = this, objectType = model_1.Model.Share, context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: stack[columnIndex - 1],
            isCreatePrevented: true,
            path: stack[columnIndex - 1].path + '/create'
        };
        return Promise.all([
            this.volumeRepository.listVolumes(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (volumes, uiDescriptor) {
            var volume = _.find(volumes, { id: volumeId });
            context.userInterfaceDescriptor = uiDescriptor;
            return Promise.all([
                self.shareRepository.getNewShare(volume, 'smb'),
                self.shareRepository.getNewShare(volume, 'nfs'),
                self.shareRepository.getNewShare(volume, 'afp'),
                self.shareRepository.getNewShare(volume, 'iscsi'),
                self.shareRepository.getNewShare(volume, 'webdav')
            ]).then(function (shares) {
                shares._objectType = objectType;
                context.object = shares;
                return self.updateStackWithContext(stack, context);
            });
        });
    };
    ShareRoute.prototype.create = function (volumeId, type, stack) {
        var self = this, objectType = model_1.Model.Share, columnIndex = 3, parentContext = stack[columnIndex], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/' + type
        };
        return Promise.all([
            this.volumeRepository.listVolumes(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (volumes, uiDescriptor) {
            var share = _.find(parentContext.object, { type: type });
            share._volume = _.find(volumes, { id: volumeId });
            context.userInterfaceDescriptor = uiDescriptor;
            context.object = share;
            return self.updateStackWithContext(stack, context);
        });
    };
    return ShareRoute;
}(abstract_route_1.AbstractRoute));
exports.ShareRoute = ShareRoute;
