"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var uuid = require("uuid");
var abstract_route_1 = require("./abstract-route");
var model_1 = require("../model");
var vm_repository_1 = require("../repository/vm-repository");
var model_event_name_1 = require("../model-event-name");
var VmDatastoreRepository_1 = require("../repository/VmDatastoreRepository");
var VmSnapshotRepository_1 = require("../repository/VmSnapshotRepository");
var VmsRoute = (function (_super) {
    __extends(VmsRoute, _super);
    function VmsRoute(vmRepository, vmDatastoreRepository, vmSnapshotRepository) {
        var _this = _super.call(this) || this;
        _this.vmRepository = vmRepository;
        _this.vmDatastoreRepository = vmDatastoreRepository;
        _this.vmSnapshotRepository = vmSnapshotRepository;
        return _this;
    }
    VmsRoute.getInstance = function () {
        if (!VmsRoute.instance) {
            VmsRoute.instance = new VmsRoute(vm_repository_1.VmRepository.getInstance(), VmDatastoreRepository_1.VmDatastoreRepository.getInstance(), VmSnapshotRepository_1.VmSnapshotRepository.getInstance());
        }
        return VmsRoute.instance;
    };
    VmsRoute.prototype.loadSection = function () {
        this.enterSection('vms');
    };
    VmsRoute.prototype.getSettings = function () {
        this.loadSettings('vms');
    };
    VmsRoute.prototype.getVm = function (vmId) {
        var columnIndex = 1;
        return this.getVmWithIdAtColumnIndex(vmId, columnIndex);
    };
    VmsRoute.prototype.createVm = function () {
        var _this = this;
        var self = this, objectType = model_1.Model.Vm, columnIndex = 1, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.vmRepository.getNewVm(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (vm, uiDescriptor) {
            context.object = vm;
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.getReadme = function () {
        var _this = this;
        var objectType = model_1.Model.VmReadme, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/readme'
        };
        return Promise.all([
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (uiDescriptor) {
            context.object = parentContext.object._readme;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.listDevices = function () {
        var _this = this;
        var objectType = model_1.Model.VmDevice, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/devices'
        };
        return Promise.all([
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (uiDescriptor) {
            var vm = parentContext.object, devices = _.map(_.reject(vm.devices, { type: _this.vmRepository.DEVICE_TYPE.VOLUME }), function (device) { return _.assign(device, {
                _objectType: model_1.Model.VmDevice,
                _vm: vm,
                id: uuid.v4()
            }); });
            context.object = devices;
            context.object._vm = vm;
            context.object._objectType = model_1.Model.VmDevice;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = _this.eventDispatcherService.addEventListener('VmDevicesChanged-' + (vm.id || 'new'), function (newDevices) {
                context.object = _.map(_.reject(newDevices, { type: _this.vmRepository.DEVICE_TYPE.VOLUME }), function (device) { return _.assign(device, {
                    _objectType: model_1.Model.VmDevice,
                    _vm: vm,
                    id: uuid.v4()
                }); });
                context.object._vm = vm;
                context.object._objectType = model_1.Model.VmDevice;
            });
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.getDevice = function (vmId, deviceId) {
        var _this = this;
        deviceId = deviceId || vmId;
        var objectType = model_1.Model.VmDevice, columnIndex = 3, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/vm-device/_/' + encodeURIComponent(deviceId)
        };
        return Promise.all([
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (uiDescriptor) {
            context.object = _.find(parentContext.object, { id: deviceId });
            context.object._vm = parentContext.object._vm;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.selectNewDeviceType = function () {
        var _this = this;
        var objectType = model_1.Model.VmDevice, columnIndex = 3, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            isCreatePrevented: true,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            Promise.all(_.map(_.values(this.vmRepository.DEVICE_TYPE), function (type) { return _this.vmRepository.getNewVmDeviceForType(type); })),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (newVmDevices, uiDescriptor) {
            var vm = parentContext.object._vm;
            context.object = _.compact(newVmDevices);
            context.object._vm = vm;
            parentContext.object = _.map(_.reject(vm.devices, { type: _this.vmRepository.DEVICE_TYPE.VOLUME }), function (device) { return _.assign(device, {
                _objectType: objectType,
                _vm: vm,
                id: uuid.v4()
            }); });
            parentContext.object._vm = vm;
            parentContext.object._objectType = objectType;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.createDevice = function (vmId, deviceType) {
        var _this = this;
        deviceType = deviceType || vmId;
        var objectType = model_1.Model.VmDevice, columnIndex = 3, parentContext = this.stack[columnIndex], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/' + encodeURIComponent(deviceType)
        };
        return Promise.all([
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (uiDescriptor) {
            context.object = _.find(parentContext.object, { _tmpId: deviceType });
            context.object._vm = parentContext.object._vm;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.listVolumes = function () {
        var _this = this;
        var objectType = model_1.Model.VmVolume, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/volumes'
        };
        return Promise.all([
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (uiDescriptor) {
            var vm = parentContext.object._vm;
            context.object = _.map(_.reject(_.filter(parentContext.object.devices, { type: _this.vmRepository.DEVICE_TYPE.VOLUME }), { properties: { type: 'NFS' } }), function (volume) { return _.assign(volume, {
                _objectType: objectType,
                _vm: vm,
                id: uuid.v4()
            }); });
            context.object._vm = parentContext.object;
            context.object._objectType = objectType;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.getVolume = function (vmId, volumeId) {
        var _this = this;
        volumeId = volumeId || vmId;
        var objectType = model_1.Model.VmVolume, columnIndex = 3, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/vm-volume/_/' + encodeURIComponent(volumeId)
        };
        return Promise.all([
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (uiDescriptor) {
            context.object = _.find(parentContext.object, { id: volumeId });
            context.object._vm = parentContext.object._vm;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.createVolume = function () {
        var _this = this;
        var objectType = model_1.Model.VmVolume, columnIndex = 3, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.vmRepository.getNewVmVolume(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (vmvolume, uiDescriptor) {
            context.object = vmvolume;
            context.object._vm = parentContext.object._vm;
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.listDatastores = function () {
        var columnIndex = 1;
        return this.loadListInColumn(this.stack, columnIndex, columnIndex - 1, '/vm-datastore', model_1.Model.VmDatastore, this.vmDatastoreRepository.list());
    };
    VmsRoute.prototype.getDatastore = function (datastoreId) {
        var _this = this;
        var objectType = model_1.Model.VmDatastore, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/vm-datastore/_/' + encodeURIComponent(datastoreId)
        };
        return Promise.all([
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (uiDescriptor) {
            context.object = _.find(parentContext.object, { id: datastoreId });
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.selectNewDatastoreType = function () {
        var _this = this;
        var objectType = model_1.Model.VmDatastore, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            isCreatePrevented: true,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            Promise.all(_.map(_.values(this.vmRepository.DATASTORE_TYPE), function (type) { return _this.vmDatastoreRepository.getNewVmDatastoreForType(type); })),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (vmdatastores, uiDescriptor) {
            context.object = _.compact(vmdatastores);
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.createDatastore = function (datastoreType) {
        var _this = this;
        var objectType = model_1.Model.VmDatastore, columnIndex = 2, parentContext = this.stack[columnIndex], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/' + encodeURIComponent(datastoreType)
        };
        return Promise.all([
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (uiDescriptor) {
            context.object = _.find(parentContext.object, { _tmpId: datastoreType });
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.listClones = function (vmId) {
        var _this = this;
        var objectType = model_1.Model.VmClone, columnIndex = 2, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/clones'
        };
        return Promise.all([
            this.vmRepository.listVms(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (vms, uiDescriptor) {
            var clones = _.sortBy(_.filter(vms, { parent: vmId }), 'name');
            clones._objectType = objectType;
            context.object = clones;
            context.userInterfaceDescriptor = uiDescriptor;
            context.changeListener = _this.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName.Vm.listChange, function (state) {
                _this.dataObjectChangeService.handleDataChange(clones, state);
                for (var i = clones.length - 1; i >= 0; i--) {
                    if (clones[i].parent !== vmId) {
                        clones.splice(i, 1);
                    }
                }
            });
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.getClone = function (vmId, cloneId) {
        var columnIndex = 3;
        return this.getVmWithIdAtColumnIndex(cloneId, columnIndex);
    };
    VmsRoute.prototype.createClone = function (vmId) {
        var _this = this;
        var objectType = model_1.Model.VmClone, columnIndex = 3, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.vmRepository.getNewVmClone(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (clone, uiDescriptor) {
            context.object = _.assign(clone, { parent: vmId });
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.listSnapshots = function (vmId) {
        var columnIndex = 2, parentContext = this.stack[columnIndex - 1];
        return this.loadListInColumn(this.stack, columnIndex, columnIndex - 1, '/snapshots', model_1.Model.VmSnapshot, this.vmSnapshotRepository.list(), {
            filter: { parent: { id: vmId } },
            sort: 'name'
        });
    };
    VmsRoute.prototype.getSnapshot = function (vmId, snapshotId) {
        var columnIndex = 3;
        return this.loadObjectInColumn(this.stack, columnIndex, columnIndex - 1, abstract_route_1.AbstractRoute.getObjectPathSuffix(model_1.Model.VmSnapshot, snapshotId), model_1.Model.VmSnapshot, this.vmSnapshotRepository.list(), { id: snapshotId });
    };
    VmsRoute.prototype.createSnapshot = function (vmId) {
        var _this = this;
        var objectType = model_1.Model.VmSnapshot, columnIndex = 3, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create'
        };
        return Promise.all([
            this.vmSnapshotRepository.getNewVmSnapshot(),
            this.vmRepository.listVms(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (snapshot, allVms, uiDescriptor) {
            context.object = _.assign(snapshot, { parent: vmId });
            context.object._parent = _.find(allVms, { id: context.object.parent });
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    VmsRoute.prototype.getVmWithIdAtColumnIndex = function (vmId, columnIndex) {
        var _this = this;
        var objectType = model_1.Model.Vm, parentContext = this.stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: objectType,
            parentContext: parentContext,
            path: parentContext.path + '/vm/_/' + encodeURIComponent(vmId)
        };
        return Promise.all([
            this.vmRepository.listVms(),
            this.modelDescriptorService.getUiDescriptorForType(objectType)
        ]).spread(function (allVms, uiDescriptor) {
            context.object = _.find(allVms, { id: vmId });
            context.object._parent = _.find(allVms, { id: context.object.parent });
            context.userInterfaceDescriptor = uiDescriptor;
            return _this.updateStackWithContext(_this.stack, context);
        });
    };
    return VmsRoute;
}(abstract_route_1.AbstractRoute));
__decorate([
    abstract_route_1.Route('/vms'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "loadSection", null);
__decorate([
    abstract_route_1.Route('/vms/settings'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "getSettings", null);
__decorate([
    abstract_route_1.Route('/vms/vm/_/{vmId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "getVm", null);
__decorate([
    abstract_route_1.Route('/vms/create'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "createVm", null);
__decorate([
    abstract_route_1.Route('/vms/create/readme'),
    abstract_route_1.Route('/vms/vm/_/{vmId}/readme'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "getReadme", null);
__decorate([
    abstract_route_1.Route('/vms/create/devices'),
    abstract_route_1.Route('/vms/vm/_/{vmId}/devices'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "listDevices", null);
__decorate([
    abstract_route_1.Route('/vms/create/devices/vm-device/_/{deviceId}'),
    abstract_route_1.Route('/vms/vm/_/{vmId}/devices/vm-device/_/{deviceId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "getDevice", null);
__decorate([
    abstract_route_1.Route('/vms/create/devices/create'),
    abstract_route_1.Route('/vms/vm/_/{vmId}/devices/create'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "selectNewDeviceType", null);
__decorate([
    abstract_route_1.Route('/vms/create/devices/create/{type}'),
    abstract_route_1.Route('/vms/vm/_/{vmId}/devices/create/{type}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "createDevice", null);
__decorate([
    abstract_route_1.Route('/vms/vm/_/{vmId}/volumes'),
    abstract_route_1.Route('/vms/create/volumes'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "listVolumes", null);
__decorate([
    abstract_route_1.Route('/vms/vm/_/{vmId}/volumes/vm-volume/_/{volumeId}'),
    abstract_route_1.Route('/vms/create/volumes/vm-volume/_/{volumeId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "getVolume", null);
__decorate([
    abstract_route_1.Route('/vms/vm/_/{vmId}/volumes/create'),
    abstract_route_1.Route('/vms/create/volumes/create'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "createVolume", null);
__decorate([
    abstract_route_1.Route('/vms/vm-datastore'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "listDatastores", null);
__decorate([
    abstract_route_1.Route('/vms/vm-datastore/_/{datastoreId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "getDatastore", null);
__decorate([
    abstract_route_1.Route('/vms/vm-datastore/create'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "selectNewDatastoreType", null);
__decorate([
    abstract_route_1.Route('/vms/vm-datastore/create/{type}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "createDatastore", null);
__decorate([
    abstract_route_1.Route('/vms/vm/_/{vmId}/clones'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "listClones", null);
__decorate([
    abstract_route_1.Route('/vms/vm/_/{vmId}/clones/vm/_/{cloneId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "getClone", null);
__decorate([
    abstract_route_1.Route('/vms/vm/_/{vmId}/clones/create'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "createClone", null);
__decorate([
    abstract_route_1.Route('/vms/vm/_/{vmId}/snapshots'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "listSnapshots", null);
__decorate([
    abstract_route_1.Route('/vms/vm/_/{vmId}/snapshots/vm-snapshot/_/{snapshotId}'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "getSnapshot", null);
__decorate([
    abstract_route_1.Route('/vms/vm/_/{vmId}/snapshots/create'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], VmsRoute.prototype, "createSnapshot", null);
exports.VmsRoute = VmsRoute;
