"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var disk_repository_1 = require("../repository/disk-repository");
var volume_repository_1 = require("../repository/volume-repository");
var _ = require("lodash");
var DiskUsageService = (function () {
    function DiskUsageService(diskRepository, volumeRepository) {
        if (diskRepository === void 0) { diskRepository = disk_repository_1.DiskRepository.getInstance(); }
        if (volumeRepository === void 0) { volumeRepository = volume_repository_1.VolumeRepository.getInstance(); }
        this.diskRepository = diskRepository;
        this.volumeRepository = volumeRepository;
    }
    DiskUsageService.getInstance = function () {
        if (!DiskUsageService.instance) {
            DiskUsageService.instance = new DiskUsageService();
        }
        return DiskUsageService.instance;
    };
    DiskUsageService.prototype.listDisks = function () {
        var _this = this;
        if (!this.initialDiskAllocationPromise || this.initialDiskAllocationPromise.isRejected()) {
            this.initialDiskAllocationPromise = this.diskRepository.listDisks().then(function (disks) {
                return _this.volumeRepository.initializeDisksAllocations(_.map(disks, 'id')).then(function () { return disks; });
            });
        }
        return this.initialDiskAllocationPromise;
    };
    DiskUsageService.prototype.listAvailableDisks = function () {
        var _this = this;
        return this.listDisks().then(function () { return _this.diskRepository.listAvailableDisks(); });
    };
    return DiskUsageService;
}());
exports.DiskUsageService = DiskUsageService;
