"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var section_repository_1 = require("../../repository/section-repository");
var event_dispatcher_service_1 = require("../event-dispatcher-service");
var data_object_change_service_1 = require("../data-object-change-service");
var AbstractSectionService = (function () {
    function AbstractSectionService() {
        this.dataObjectChangeService = new data_object_change_service_1.DataObjectChangeService();
        this.eventDispatcherService = event_dispatcher_service_1.EventDispatcherService.getInstance();
        var self = this, initReturn = this.init();
        if (!Promise.is(initReturn)) {
            initReturn = Promise.resolve();
        }
        this.instanciationPromise = initReturn.then(function () {
            return self.load();
        });
    }
    AbstractSectionService.prototype.findObjectWithId = function (entries, id) {
        for (var _i = 0, entries_1 = entries; _i < entries_1.length; _i++) {
            var entry = entries_1[_i];
            if (entry.id === id) {
                return entry;
            }
        }
        return null;
    };
    AbstractSectionService.prototype.load = function () {
        var _this = this;
        return Promise.all([
            AbstractSectionService.sectionRepository.getNewSection(),
            AbstractSectionService.sectionRepository.getNewSectionSettings(),
            this.loadEntries(),
            this.loadExtraEntries(),
            this.loadSettings(),
            this.loadOverview()
        ]).spread(function (section, sectionSettings, entries, extraEntries, settings, overview) {
            sectionSettings.section = section;
            sectionSettings.settings = settings;
            _this.section = section;
            _this.section.settings = sectionSettings;
            _this.entries = _this.section.entries = entries;
            _this.extraEntries = _this.section.extraEntries = extraEntries;
            _this.overview = _this.section.overview = overview;
            return _this;
        });
    };
    return AbstractSectionService;
}());
AbstractSectionService.sectionRepository = section_repository_1.SectionRepository.getInstance();
exports.AbstractSectionService = AbstractSectionService;
