var AbstractInspector = require("ui/abstract/abstract-inspector").AbstractInspector;

exports.EncryptedVolumeImporter = AbstractInspector.specialize({
    enterDocument: {
        value: function(isFirstTime) {
            if (isFirstTime) {
                this.addRangeAtPathChangeListener("object.disks", this, "_handleDisksChange");
            }
            var self = this;
            this._sectionService.listAvailableDisks().then(function(availableDisks) {
                self.availableDisks = availableDisks;
            });
            this._sectionService.clearReservedDisks();
        }
    },

    save: {
        value: function() {
            this._sectionService.importEncryptedVolume(this.object);
        }
    },

    shouldAcceptComponentInDiskCategory: {
        value: function(component, diskCategory) {
            return component.object &&
                    diskCategory.disks.indexOf(component.object) === -1 &&
                    component.object.status &&
                    component.object.status.is_ssd === diskCategory.isSsd;
        }
    },

    handleComponentDropInDiskCategory: {
        value: function(component) {
            var disk = component.object,
                diskIndex = this.object.disks.indexOf(disk);
            if (diskIndex !== -1) {
                this.object.disks.splice(diskIndex, 1);
            }
        }
    },

    handleImportVolumeAction: {
        value: function () {
            this._sectionService.importEncryptedVolume(this.object.name, this.object.disks, this.object.key, this.object.password);
        }
    },

    _handleDisksChange: {
        value: function(addedDisks, removedDisks) {
            var i, length;
            for (i = 0, length = addedDisks.length; i < length; i++) {
                this._sectionService.markDiskAsReserved(addedDisks[i].path);
            }
            for (i = 0, length = removedDisks.length; i < length; i++) {
                this._sectionService.markDiskAsNonReserved(removedDisks[i].path);
            }
        }
    }
});
